/* *
 * @file       groestl_tb.c
 * @brief      Testbench used for HLS-based Groestl in software
 * @project    ARC-2015 HLS SHA-3
 * @author     Ekawat (ice) Homsirikamol
 * @version    1.0
 * @copyright  Copyright (c) 2014 Cryptographic Engineering Research Group
 *             ECE Department, George Mason University Fairfax, VA, U.S.A.
 *             All rights Reserved.
 * @license    This project is released under the GNU Public License.
 *             The license and distribution terms for this file may be
 *             found in the file LICENSE in this distribution or at
 *             http://www.gnu.org/licenses/gpl-3.0.txt
 * @note       This is publicly available encryption source code that falls
 *             under the License Exception TSU (Technology and software-
 *             —unrestricted)
 */
 
#include <string.h>
#include <stdio.h>
#include "ap_cint.h"
#include "groestl_hls.h"

#define TEST1 1

void printData(u8 *data, u8 size)
{
  int i;
  for (i=0; i<size; i++)
  {
    printf("%02X", data[i]);
    if ((i+1)%8 == 0) printf(" ");
    if ((i+1)%32 == 0) printf("\n");
  }
  printf("\n");
}

int testGroestl() {
  int i;
  int pass = 1;
  u32 size;
#if TEST1
  u8 input[64] = {
      0xCC, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01
  };
  u8 exp[32]={
      0x15, 0xE2, 0x67, 0x1F, 0x0E, 0xAF, 0x66, 0xC0,
      0xDE, 0x30, 0x93, 0xAB, 0x7B, 0x1E, 0x39, 0xDC,
      0x68, 0xF9, 0x45, 0xD7, 0x00, 0x2F, 0xC5, 0xDF,
      0xD5, 0x2D, 0x60, 0x52, 0x7E, 0x72, 0x28, 0xD1
  };
#else
  u8 input[64] = {
      0x42, 0xE9, 0x9A, 0x2F, 0x80, 0xAE, 0xE0, 0xE0,
      0x01, 0x27, 0x9A, 0x24, 0x34, 0xF7, 0x31, 0xE0,
      0x1D, 0x34, 0xA4, 0x4B, 0x1A, 0x81, 0x01, 0x72,
      0x69, 0x21, 0xC0, 0x59, 0x0C, 0x30, 0xF3, 0x12,
      0x0E, 0xB8, 0x30, 0x59, 0xF3, 0x25, 0xE8, 0x94,
      0xA5, 0xAC, 0x95, 0x9D, 0xCA, 0x71, 0xCE, 0x22,
      0x14, 0x79, 0x99, 0x16, 0x42, 0x4E, 0x85, 0x9D,
      0x27, 0xD7, 0x89, 0x43, 0x7B, 0x9D, 0x27, 0x24
  };
  u8 input2[64] = {
      0x0B, 0xF8, 0xC3, 0x5A, 0xDB, 0xAF, 0xCE, 0xCC,
      0x32, 0x2B, 0x48, 0xAA, 0x20, 0x5B, 0x29, 0x39,
      0x62, 0xD8, 0x58, 0x65, 0x2A, 0xBA, 0xCB, 0xD5,
      0x88, 0xBC, 0xF6, 0xCB, 0xC3, 0x88, 0xD0, 0x99,
      0x3B, 0xD6, 0x22, 0xF9, 0x6E, 0xD5, 0x46, 0x14,
      0xC2, 0x5B, 0x6A, 0x9A, 0xA5, 0x27, 0x58, 0x9E,
      0xAA, 0xFF, 0xCF, 0x17, 0xDD, 0xF7, 0x80, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02
  };
  u8 exp[32]={
      0x01, 0x93, 0x63, 0x36, 0x94, 0xB5, 0x32, 0x96,
      0xDA, 0x8F, 0x1D, 0xFD, 0xE3, 0x78, 0x34, 0x66,
      0x80, 0x4C, 0x1D, 0xC1, 0xD9, 0x5E, 0x89, 0x55,
      0x5E, 0x06, 0x92, 0x28, 0x54, 0x4A, 0xEF, 0xB6
  };
#endif
  u8 output[32];



  for (i=0; i<32; i++)
    output[i] = 0;

  printf("Pre-hash:\n");
  printData(output, 32);
  printf("Input:\n");
  printData(input, 64);
#if TEST1
  groestl(input, output, 1);
#else  
  groestl(input, output, 0);
  groestl(input2, output, 1);
#endif
  printf("Post-hash:\n");
  printData(output, 32);
  printf("Expect:\n");
  printData(exp, 32);

  /* Check result */
  for(i=0; i<32; i++)
    if (exp[i] != output[i]) {printf("Test fail at word %d\n", i); pass = 0;}
  if (pass == 1)
#if TEST1
    printf("Hooray! One block test passed\n");
#else
    printf("Hooray! Two block test passed\n");
#endif



  return 0;
}

int main() {
  testGroestl();
  return 0;
}
